/* $Workfile: OsaLinuxC.c $   $Revision: 4 $    $Modtime: 4/02/13 12:46p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2013, All Rights Reserved

*******************************************************************************/

#include <linux/version.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/semaphore.h>
#include <linux/pci.h>

#include "OsaLinuxC.h"
#include "OsaLinuxRtl.h"
#include "OsaLinuxCpp.h" 
#include "OsaRtl.h"

static OsaUint32 g_LogLevel = 1; // log errors only

/*---------------------------------------------------------------------------*\  
 from OsaLinuxCppRtl.h OsaLinuxCppRtl.cpp
\*---------------------------------------------------------------------------*/
void OsaSpinLockImpl_Ctor(void **pSLock)
{
  if (sizeof(spinlock_t) > 0) {

    if(in_interrupt() || irqs_disabled())
     *pSLock = kmalloc(sizeof(spinlock_t), GFP_ATOMIC);
    else *pSLock = kmalloc(sizeof(spinlock_t), GFP_KERNEL);

    if (*pSLock) {
      spin_lock_init((spinlock_t *) *pSLock);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaSpinLockImpl_Ctor!\n"));
    }
  } else {
    spin_lock_init((spinlock_t *) *pSLock);
  }
}

void OsaSpinLockImpl_Dtor(void **pSLock)
{
  if (sizeof(spinlock_t) > 0) {
    if (*pSLock) {
      kfree(*pSLock);
      *pSLock = NULL;
    }
  }
}

void OsaSpinLockImpl_Lock(void *pSLock, OsaULong *flags)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32)
  __u32 lflags;
#else  //2.6.32
  OsaULong lflags;
#endif

  spin_lock_irqsave((spinlock_t *) pSLock, lflags);
  *flags = lflags;
}

void OsaSpinLockImpl_Unlock(void *pSLock, OsaULong savedFlags)
{
  spin_unlock_irqrestore((spinlock_t *) pSLock, (OsaULong) savedFlags);
}

void OsaEventImpl_Ctor(void **pWQHead)
{
  if (sizeof(wait_queue_head_t) > 0) {

    if(in_interrupt() || irqs_disabled())
     *pWQHead = kmalloc(sizeof(wait_queue_head_t), GFP_ATOMIC);
    else *pWQHead = kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);

    if (*pWQHead) {
      init_waitqueue_head((wait_queue_head_t *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaEventImpl_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue_head_t *) *pWQHead);
  }

}

void OsaEventImpl_Dtor(void **pWQHead)
{
  if (*pWQHead) {

    if (sizeof(wait_queue_head_t) > 0) {
      kfree(*pWQHead);
    }

  }
  *pWQHead = NULL;
}


void OsaEventImpl_wake_up(void **pWQHead)
{
  wake_up((wait_queue_head_t *) *pWQHead);
}


void OsaSemaphoreImpl_Ctor(void **pSemaphore)
{

  if(in_interrupt() || irqs_disabled())
     *pSemaphore = kmalloc(sizeof(struct semaphore), GFP_ATOMIC);
  else *pSemaphore = kmalloc(sizeof(struct semaphore), GFP_KERNEL);

  if (*pSemaphore) {
    sema_init((struct semaphore *) *pSemaphore, 0);     // initial count 0
  } else {
    OSA_EPRINTF(("OsaLinux: Out of memory OsaSemaphoreImpl_Ctor!\n"));
  }
}

void OsaSemaphoreImpl_Dtor(void **pSemaphore)
{
  if (*pSemaphore) {
    kfree(*pSemaphore);
    *pSemaphore = NULL;
  }
}

void OsaSemaphoreImpl_Release(void **pSemaphore)
{
  up((struct semaphore *) *pSemaphore);
}

void OsaSemaphoreImpl_WaitForSemaphore(void **pSemaphore)
{
  down((struct semaphore *) *pSemaphore);
}


/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/
  
void OsaLinuxUserEventDevice_Ctor(void **pWQHead)
{

  if (sizeof(wait_queue_head_t) > 0) {
    *pWQHead = kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);
    if (*pWQHead) {
      init_waitqueue_head((wait_queue_head_t *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaLinuxUserEventDevice_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue_head_t *) *pWQHead);
  }

}

void OsaLinuxUserEventDevice_Dtor(void **pWQHead)
{
  if (*pWQHead) {

    if (sizeof(wait_queue_head_t) > 0) {
      kfree(*pWQHead);
    }

  }
  *pWQHead = NULL;
}

void OsaLinuxUserEventDevice_wake_up(void **pWQHead)
{
  wake_up((wait_queue_head_t *) *pWQHead);
}

/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/

int OsaLinux_pci_present()
{
#ifdef CONFIG_PCI
  return 1;
#else
  return 0;
#endif
}


void OsaLinux_pci_read_config_word(void *dev, int where, u16 *val)
{
  pci_read_config_word((struct pci_dev *)dev, where, val);
}


void OsaLinux_pci_read_config_dword(void *dev, int where, u32 *val)
{
  pci_read_config_dword((struct pci_dev *)dev, where, val);
}


void OsaLinux_pci_write_config_dword(void *dev, int where, u32 val)
{
  pci_write_config_dword((struct pci_dev *)dev, where, val);
}

void * OsaLinux_pci_dev_next(void *from)
{
  return (void*)pci_get_device(PCI_ANY_ID, PCI_ANY_ID, from);
}


OsaUint16 OsaLinux_pci_dev_device(void *dev)
{
 return ((struct pci_dev*)dev)->device;
}

OsaUint16 OsaLinux_pci_dev_vendor(void *dev)
{
 return ((struct pci_dev*)dev)->vendor;
}

unsigned int OsaLinux_pci_dev_irq(void *dev)
{
 return ((struct pci_dev*)dev)->irq;
}

unsigned int OsaLinux_pci_dev_resource_start(void *dev, int id)
{
 return ((struct pci_dev*)dev)->resource[id].start;
}


void OsaLinux_pci_dev_enable(void *d)
{
  int iReturn; 

  struct pci_dev *dd = d;
  iReturn = pci_enable_device(dd);

}


OsaInt32 OsaGetKernelStackAvailable(OsaULong *stackAvailable)
{
  static OsaULong lowestStackValue = (12*1024); // State with large number
  
  OsaUint64 esp;

//todo:
#if 0
  __asm__ __volatile__("andl %%esp,%0" :
     "=r" (esp) : "0" (THREAD_SIZE - 1));
#else
  esp = 1024*4;
#endif

  // Unused stack
  *stackAvailable = esp;

  if(lowestStackValue > esp) {
    lowestStackValue = esp;
    // OSA_DPRINTF will/could call OsaGetKernelStackAvailable()
    printk(KERN_INFO "**** OsaLinux: Current lowest stack value = %lx\n", (OsaULong) lowestStackValue);
  }

  // Stack size max is 4K
  // During driver initialization 900 is reached with Vidar 55x. Then
  // the stack usage drops well below 1000. Set the upper limit 
  // high enough so that only if the stack is 75% used.
  if(*stackAvailable < 1000) { 
    // OSA_DPRINTF will/could call OsaGetKernelStackAvailable()
    printk(KERN_INFO "\nOsaLinux: Stack is about to overflow, stack available size = %lx\n", *stackAvailable);
		
    // the following two lines are commented out, they cause stack overflow in CentOS 5.2
    //dump_stack(); 
    //printk(KERN_INFO "OsaLinux: end of dump_stack\n");
  }
   
  return OSA_RES_SUCCESS;

}

/******************************************************************************
    void
    OsaSetLogErrorEvent(IN OsaUint32 nLogLevelFactor)

******************************************************************************/
void
OsaSetLogErrorEvent(
    IN OsaUint32 nLogLevelFactor
    )
{
  g_LogLevel = nLogLevelFactor;
}

/******************************************************************************
    OsaUint32
    OsaGetLogErrorEvent(void)

******************************************************************************/
OsaUint32
OsaGetLogErrorEvent(
    void
    )
{
  return(g_LogLevel);
}

/******************************************************************************
    OsaUint32
    OsaLogErrorEvent(
        IN OsaUint32 ErrorCode,
        IN OsaUint32 UniqueID ,
        IN OsaUint32 DeviceType,
        IN char      String1  ,
        IN OsaUint16 String2  
        )

    IN OsaUint32 ErrorCode  Error code
    IN OsaUint32 UniqueID   Unique error ID, used when there is more than one 
                            place reporting the same ErrorCode. It is usually 
                            the OTX Result Code.
    IN OsaUint32 DeviceType OtxPhysicalDeviceTypeE or 0 (OTX_DEVICE_UNKNOWN) 
    IN char      String1    Optional substitution string 1.
    IN OsaUint16 String1    Optional substitution string 2.

    return Otx_Result
******************************************************************************/
OsaUint32
OsaLogErrorEvent(
    IN OsaUint32 ErrorCode,
    IN OsaUint32 UniqueID,
    IN OsaUint32 DeviceType,
    IN char      *String1,
    IN OsaUint16 *String2)
{

#if DBG
  // Debug mode log additional messages.
#else
    // Only log events for error status "OTX_FAILURE" and informational events
  if(((UniqueID & 0x80000000) == 0) && (ErrorCode != OSA_INFORMATIONAL_EVENT_LOG)) {
    // Not an error
    return(UniqueID);
  }
#endif

  if((OsaGetCurrentExeLevel() == OSA_EXE_LEVEL_DEVICE) &&
     (ErrorCode != OSA_INFORMATIONAL_EVENT_LOG)) {
    // Interrupt handler for a device. Adding messages to log file could slow
    // processing of interrupt.
    return(UniqueID);
  }

  if(String1 && (String1[1] != 0)) {
    if(UniqueID) {
      printk(KERN_INFO "Otx Error Event: %s, Result Code 0x%x\n", String1, UniqueID);
    }
    else {
      printk(KERN_INFO "Otx Info Event: %s, Result Code 0x%x\n", String1, UniqueID);
    }
  }
  else {
    if(UniqueID) {
     printk(KERN_INFO "Otx: No error string available, Result Code 0x%x\n", UniqueID);
    }
  }

  return(UniqueID);
}


/* -------------------------------------------------------------------------------------- */

/*
 * stubs called by C++ code
 *
 * we basically "hide" any code & headers which require including kernel headers
 */

int osalinux_printk(const char *fmt, ...)
{
  va_list args;
  int r;
  va_start(args, fmt);
  r = vprintk(fmt, args);
  va_end(args);

#if 1
  /*64bit - for entry/exit calls*/
  {
    int i;
    for (i = 0; fmt[i]; i++)
      if (fmt[i] == '%' || fmt[i] == '\n') return r;

    /* fmt string has no args and no newline */
    printk("\n");
  }
#endif

  return r;
}

void osalinux_atomic_add(int i, osalinux_atomic_t *v)
{
  atomic_add(i, (atomic_t *)v);
}

void osalinux_atomic_inc(osalinux_atomic_t *v)
{
  atomic_inc((atomic_t *)v);
}

void osalinux_atomic_dec(osalinux_atomic_t *v)
{
  atomic_dec((atomic_t *)v);
}

int osalinux_atomic_read(const osalinux_atomic_t *v)
{
  return atomic_read((atomic_t *)v);
}

void osalinux_atomic_set(osalinux_atomic_t *v, int i)
{
  atomic_set((atomic_t *)v, i);
}

int osalinux_check_region(unsigned long s, unsigned long n)
{
  return check_region((resource_size_t)s, (resource_size_t)n);
}

void *osalinux_request_region(unsigned long start, unsigned long n, const char *name)
{
  return request_region((resource_size_t)start, (resource_size_t)n, name);
}

void osalinux_release_region(unsigned long start, unsigned long n)
{
  release_region((resource_size_t)start, (resource_size_t)n);
}

 
