/* $Workfile: OsaLinuxMemory.cpp $   $Revision: 2 $    $Modtime: 4/01/13 4:17p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2013, All Rights Reserved

*******************************************************************************/

#include "Api/OsaDef.h"
#include "../OsaRtl.h"
#include "../OsaMemory.h"
#include "../OsaMemDbg.h"

#define EXTRA_DPRINTF 1      // set to 1 for more debug printouts
       
#ifdef new
#undef new
#endif

#if DBG
/******************************************************************************
******************************************************************************/
void * operator new(size_t cbSize, char *szFileName, int nLine)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED, szFileName, nLine);
}


/******************************************************************************
******************************************************************************/
void * operator new[](size_t cbSize, char *szFileName, int nLine)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED, szFileName, nLine);
}


/******************************************************************************
******************************************************************************/
void * operator new(size_t cbSize, OsaMemoryPoolType type, char *szFileName, int nLine)
{
  OSA_ASSERT(cbSize);  // Normally it is possible to allocate 0 bytes in C++ but not with ExAllocatePool()
  void *ptr;
  
  ptr = osalinux_kmalloc(cbSize + OSA_EXTRA_MEM_NEEDED);

  if (ptr) {

    ptr = OsaMakeMemoryHeaderTail(ptr, cbSize, szFileName, nLine);

    if (type == OSA_MEMORY_PAGED) {
      OSA_DPRINTF(("OsaLinux: Memory is Paged!\n"));
    }

  } else {
    (*__newhandler)(cbSize);  
  }

  OSA_ASSERT(ptr);  // Since we might not always check memory allocation return values, make sure that we know when memory allocation failed
  return ptr;
}

#endif


/******************************************************************************
******************************************************************************/
void *operator new(size_t cbSize)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED);
}


/******************************************************************************
******************************************************************************/
void *operator new[](size_t cbSize)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED);
}


/******************************************************************************
******************************************************************************/
void *operator new(size_t cbSize, OsaMemoryPoolType type)
{

  OSA_ASSERT(cbSize);  // Normally it is possible to allocate 0 bytes in C++ but not with ExAllocatePool()
  void *ptr;

  ptr = osalinux_kmalloc(cbSize + OSA_EXTRA_MEM_NEEDED);

  if (ptr) {

#if DBG
    ptr = OsaMakeMemoryHeaderTail(ptr, cbSize, "Unknown Name", 0);

    if (type == OSA_MEMORY_PAGED) {
      OSA_DPRINTF(("OsaLinux: Memory is Paged!\n"));
    }
#endif

  } else {
    (*__newhandler)(cbSize);  
  }

  OSA_ASSERT(ptr);  // Since we might not always check memory allocation return values, make sure that we know when memory allocation failed
  return ptr;
}


/******************************************************************************
******************************************************************************/
void * operator new(size_t cbSize, OSA_MD& Omd)
{
  if (Omd.cbSize != cbSize) {
    Omd.cbSize = cbSize;
  }
  Omd.pPhy = ::operator new(Omd.cbSize, OSA_MEMORY_NONPAGED);
  if (Omd.pPhy) {
    Omd.pContext = (void *) NULL;
  } else {
    Omd.pContext = (void *) NULL;
  }

  return Omd.pPhy;
}

/******************************************************************************
******************************************************************************/
void operator delete(void *ptr, OSA_MD& Omd)
{
  if (Omd.pContext) {
    Omd.pContext = (void *) NULL;
  }

  ::operator delete(Omd.pPhy);
  Omd.pPhy = (void *) NULL;
}

/******************************************************************************
******************************************************************************/
void operator delete(void *ptr)
{
  //OSA_DPRINTF(("OsaLinux: Deleting ptr = %p\n", ptr));
  //OSA_ASSERT(NULL != p);

#if DBG
  if (ptr) {
    void *pUser = ptr;   // Goofy assignments back and forth solely to take the speed hit in the DBG version only
    ptr = OsaUndoMemoryHeaderTail(pUser);
  }
#endif

  if (ptr) {
    osalinux_kfree(ptr);
  }
  else {
    OSA_DPRINTF(("OsaLinux::delete(ptr = 0) nothing to deleting\n"));
  }
}

/******************************************************************************
******************************************************************************/
void operator delete[](void *ptr)
{
  //OSA_DPRINTF(("OsaLinux: Ptr = %p\n", ptr));
  //OSA_ASSERT(NULL != p);

  delete ptr;
}


