/* $Workfile: OsaLinuxRtl.h $   $Revision: 3 $    $Modtime: 4/01/13 5:58p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2013, All Rights Reserved

*******************************************************************************/

#ifndef OSA_LINUX_RTL_H
#define OSA_LINUX_RTL_H

#include <linux/version.h> // for LINUX_VERSION_CODE, KERNEL_VERSION 
#include "../OsaDef.h"


#if DBG

// Setting this value to 1 will install a stack check into the OSA_DPRINTF
// that will dump a call stack track if the stack is less then 1000 bytes.
//64bit: #define STACK_CHECK_ENABLED 1


#if STACK_CHECK_ENABLED == 1

#ifdef __cplusplus
extern "C" {
#endif

extern OsaInt32 OsaGetKernelStackAvailable(OsaULong *stackAvailable);
#define OSA_DPRINTF(pars)     { {OsaULong stackAvailable; OsaGetKernelStackAvailable(&stackAvailable);} osalinux_printk pars;}
#define OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable

#ifdef __cplusplus
}
#endif

#else // not STACK_CHECK_ENABLED == 1

#define OSA_DPRINTF(pars)     osalinux_printk pars
#ifdef OSA_DUMP_CALLSTACK
#undef  OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable    
#endif

#endif  // end of STACK_CHECK_ENABLED == 1


#define OSA_EPRINTF(pars)     { osalinux_printk("%s(%d) : ", __FILE__, __LINE__); osalinux_printk pars; }
#define OSA_BREAKPOINT()
#define OSA_ASSERT(condition) { if (!(condition)) { osalinux_printk("%s(%d) : ASSERT(%s) FAILED\n", __FILE__, __LINE__, #condition); } }
#define OSA_FATAL(x)          { osalinux_printk("%s(%d) : FATAL ERROR\n", __FILE__, __LINE__); }

#else // no DBG

#ifdef OSA_DUMP_CALLSTACK
#undef  OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable    
#endif
#define OSA_DPRINTF(x)
#define OSA_EPRINTF(pars) { osalinux_printk("%s(%d) : ", __FILE__, __LINE__); osalinux_printk pars; }
#define OSA_BREAKPOINT()
#define OSA_ASSERT(x)
#define OSA_FATAL(x)

#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __cplusplus
#ifdef __x86_64
typedef unsigned int wchar_t;		/* sizeof(wchar_t) == 4 */
#else
  typedef unsigned long wchar_t;
#endif
#endif

// In Linux a user event exists as a struct in user space and as a device in OSA, in the driver we are just allowed
  // to refer to it using a opaque pointer
typedef void *OsaUserEventP;

typedef osalinux_atomic_t OsaInterlockedInt, *OsaInterlockedIntP;

void OsaInitLinuxRtl(void);

#define OsaInterlockedRead(v)     osalinux_atomic_read(v)
#define OsaInterlockedWrite(v, i) osalinux_atomic_set(v, i)
#define OsaAddrOfFunc(f) (&f)

int osalinux_wait_event_timeout(OsaUint32 *pEventSet, void **ppWaitQueue, OsaUint32 nWaitMaxMs);

void *osalinux_kmalloc(size_t size);
void osalinux_kfree(void *ptr);

int osalinux_request_irq(unsigned int irq, void *dev_id);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32)
void OsaLinuxInterruptCallback(int irq, void *context, struct pt_regs *regs);
#else //2.6.32
void OsaLinuxInterruptCallback(int irq, void *context);
#endif

void osalinux_free_irq(unsigned int irq, void *dev_id);
void *osalinux_ioremap(OsaULong offset, OsaULong size);
void osalinux_iounmap(void *addr);

void osalinux_wake_up(void **wait_queue);
void osalinux_poll_wait(void * filp, void **wait_address, void *p);

int osalinux_check_region(unsigned long s, unsigned long n);
void *osalinux_request_region(unsigned long start, unsigned long n, const char *name);
void osalinux_release_region(unsigned long start, unsigned long n);

size_t  wcslen(const wchar_t *psz);
wchar_t *wcscpy(wchar_t *strDestination, const wchar_t *strSource);
int wcscpy_s(wchar_t *strDestination, size_t cchDest, const wchar_t *strSource);
int strcat_s(char *dest, size_t cchDest, const char *source);
wchar_t *wcsncpy(wchar_t *strDest, const wchar_t *strSource, size_t count);
int wcsncpy_s(wchar_t *strDest, size_t cchDest, const wchar_t *strSource, size_t count);
int strncpy_s(char *dest, size_t cchDest, const char *source, size_t count);
wchar_t towupper(wchar_t c);
char *wcsztosz(char *strDest, const wchar_t *strSource);
char *_strrev(char *szBuf);

OsaUint32 getbits(OsaUint32 x, int p, int n);

// inline functions
// For some reason related to C/C++ inclusion of header file I could not include the asm/io.h header
// Therefore we use macros instead for these low level functions.
#if 1 // I could not get the macro to compile without getting the 'warning: implicit declaration of function' /HJ 2000-09-14

OsaUint32   OsaReadReg32(OsaULong pRegister);
OsaUint16   OsaReadReg16(OsaULong pRegister);
OsaUint8    OsaReadReg8 (OsaULong pRegister);

void        OsaWriteReg32(OsaULong pRegister, OsaUint32 nValue);
void        OsaWriteReg16(OsaULong pRegister, OsaUint16 nValue);
void        OsaWriteReg8 (OsaULong pRegister, OsaUint8  nValue);

// Read and write many bytes from the same memory register
void        OsaReadRegBuffer8  (OsaULong pRegister, OsaUint8  *pBuffer, OsaUint32 coCount);
void        OsaWriteRegBuffer8 (OsaULong pRegister, OsaUint8  *pBuffer, OsaUint32 coCount);

OsaUint32   OsaReadPort32(OsaULong pPort);
OsaUint16   OsaReadPort16(OsaULong pPort);
OsaUint8    OsaReadPort8 (OsaULong pPort);

void        OsaWritePort32(OsaULong pPort, OsaUint32 nValue);
void        OsaWritePort16(OsaULong pPort, OsaUint16 nValue);
void        OsaWritePort8 (OsaULong pPort, OsaUint8  nValue);

// Read and write many bytes from the same I/O port
void        OsaReadPortBuffer8 (OsaULong pPort, OsaUint8  *pBuffer, OsaUint32 coCount);
void        OsaWritePortBuffer8(OsaULong pPort, OsaUint8  *pBuffer, OsaUint32 coCount);

#else
#define OsaReadReg32(pRegister) readl(pRegister)
#define OsaReadReg16(pRegister) readw(pRegister)
#define OsaReadReg8 (pRegister) readb(pRegister)

#define OsaWriteReg32(pRegister, nValue) writel(nValue, pRegister)
#define OsaWriteReg16(pRegister, nValue) writew(nValue, pRegister)
#define OsaWriteReg8 (pRegister, nValue) writeb(nValue, pRegister)

// Read and write many bytes from the same memory register
#define OsaReadRegBuffer8  (pRegister, pBuffer, coCount)         /*
*/ {                                                             /*
*/  OsaUint32 coReads;                                           /*
*/  for (coReads = 0; coReads < coCount; coReads++, pBuffer++) { /*
*/    *pBuffer = readb(pRegister);                               /* 
*/  }                                                            /*
*/ }


#define  OsaWriteRegBuffer8 (pRegister, pBuffer, coCount)            /*
*/ {                                                                 /*
*/   OsaUint32 coWrites;                                             /*
*/   for (coWrites = 0; coWrites < coCount; coWrites++, pBuffer++) { /*
*/     writeb(*pBuffer, pRegister);                                  /*
*/   }                                                               /*
*/ }

#define OsaReadPort32(pPort) inl(pPort)
#define OsaReadPort16(pPort) inw(pPort)
#define OsaReadPort8 (pPort) inb(pPort)

#define OsaWritePort32(pPort, nValue) outl(nValue, pPort)
#define OsaWritePort16(pPort, nValue) outw(nValue, pPort)
#define OsaWritePort8 (pPort, nValue) outb(nValue, pPort)

// Read and write many bytes from the same I/O port
#define OsaReadPortBuffer8 (pPort, pBuffer, coCount) insb(pPort, pBuffer, coCount)
#define OsaWritePortBuffer8(pPort, pBuffer, coCount) outsb(pPort, pBuffer, coCount)
#endif

// DMA routines
void *osalinux_alloc_dma_common_buf(void *pPciDev, size_t size, OsaUint64 *pbus_addr);
void osalinux_free_dma_common_buf(void *pPciDev, size_t size, OsaUint64 log_addr, OsaUint64 bus_addr);

// dpc routines
void osalinux_request_deferred_board_notification(
  void (*pfnDeferredBoardNotificationCallback)(void *, void *, void *),
  void *internalDeviceRep,
  void *pIrp,
  void *pContext
);

OsaBool32 osalinux_request_delayed_dpc(
  void (*pfnDpc)(void *, void *, void *, void *),
  void *pDpcContext,
  void *pContext,
  void *pArg1,
  void *pArg2,
  OsaULong dueTimeInMicroSeconds
);

void osalinux_request_dpc(
  void (*pfnDpc)(void *, void *, void *, void *),
  void *pDpcContext,
  void *pContext,
  void *pArg1,
  void *pArg2
);

void osalinux_save_flags_cli(OsaULong *flags);
void osalinux_restore_flags(OsaULong flags);

#ifdef __cplusplus
}
#endif

#endif
