/* $Workfile: OsaImplementation.h $   $Revision: 3 $    $Modtime: 3/21/13 4:25p $  */
/*******************************************************************************

  Copyright (c) Odin TeleSystems Inc., 2008-2013, All Rights Reserved.

  OTX Telecom Adapter Driver

  @doc internal

  @module

*******************************************************************************/

#ifndef OSA_IMPLEMENTATION_H
#define OSA_IMPLEMENTATION_H

#include "Osa.h"

struct OsaDevice {
  OsaDevice()
    : m_internalDeviceRep(NULL)
    , m_bBoard(false)
    , m_eBusType(OSA_BUS_UNKNOWN)
    , m_nHwLocationAddress(0xFFFFFFFF)
    , m_nPcmciaSocketNumber(-1)
    , m_nPcmciaControllerNumber(-1)
    , m_pResources(NULL)
    , m_pfnBoardNotificationCallback(NULL)
    , m_pfnDeferredBoardNotificationCallback(NULL)
    , m_DmaMaxMapRegisters(0)
  {}

#ifdef _USB
  ~OsaDevice() { //DENIS
    if (m_pResources) {
      delete m_pResources;
    }
  }
#endif
  
  OsaInternalDeviceRep            m_internalDeviceRep;
  OsaString                       m_strName;
  bool                            m_bBoard;                                // Is it a board or not
  OsaBusTypeE                     m_eBusType;                              // PCI or PCMCIA
  OsaUint32                       m_nHwLocationAddress;                    // For a PCI board it is device number in high word and function number in low word, for PCMCIA it is socket number (0x00 or 0x40)
  OsaInt32                        m_nPcmciaSocketNumber;                   // Only used for PCMCIA boards. Socket number (0 or 1)
  OsaInt32                        m_nPcmciaControllerNumber;               // Only used for PCMCIA boards. Controller number (0 - 3)
  OsaResourcesS                   *m_pResources;                           // Only set for boards when resources are mapped and connected
  OsaBoardNotificationFnP         m_pfnBoardNotificationCallback;          // Only used for boards
  OsaDeferredBoardNotificationFnP m_pfnDeferredBoardNotificationCallback;  // Only used for boards

  OsaULong                        m_DmaMaxMapRegisters;                    // maximum number of dma map registers
};


// Function pointers for the platform independent driver entry points 
// These functions will all run at EXE_LEVEL_CONTROL which means they can be treated as
// general threads and call functions that require the "Scheduler"/"Thread Dispatcher" to be active
struct OsaDriverEntryPointsS  {
  OsaUnloadFnP                     pfnUnload;                      // @@field Called when driver unloads
  OsaAddBoardFnP                   pfnAddBoard;                    // @@field Creates the object representing the hardware device on the system bus
  OsaStartBoardFnP                 pfnStartBoard;                  // @@field Reads configuration info from board and creates remaining substructures needed
  OsaBoardStateFnP                 pfnBoardState;                  // @@field Sets the different states for IRP handling and IO
  OsaRemoveBoardFnP                pfnRemoveBoard;                 // @@field Deletes everything created in AddBoard and StartBoard
  OsaOpenDeviceFnP                 pfnOpenDevice;                  // @@field Opens one of the driver objects for reading or writing or both, may also create the device
  OsaCloseDeviceFnP                pfnCloseDevice;                 // @@field Closes an opened device, may also delete it
  OsaIoControlDeviceFnP            pfnIoControlDevice;             // @@field IoControl dispatch function
  OsaHandleInterruptFnP            pfnHandleInterrupt;             // @@field Handle interrupt function
};


void OsaInit();

bool OsaCheckCallbacks();

  // The platform independent part of the driver must set these function pointers
  // to the appropriate functions. See also <t OsaDriverEntryPointsS>
extern OsaDriverEntryPointsS  g_sOsaDriverEntryPoints; 

extern bool gOsaDriverCompatibilityMode;


#endif  // OSA_IMPLEMENTATION_H

